/*
 * Copyright (c) 2016 by ELECTRONIC ASSEMBLY <technik@lcd-module.de>
 * MCP7940 library for arduino.
 *
 * This file is free software; you can redistribute it and/or modify
 * it under the terms of either the GNU General Public License version 2
 * or the GNU Lesser General Public License version 2.1, both as
 * published by the Free Software Foundation.
 */


#include "MCP7940.h"
#include <Wire.h>

/*----------------------------
Func: initialize
Desc: Initializes RTC MCP7940
Vars: *str = buffer to print strings into
------------------------------*/
void MCP7940::initialize(char *str)
{
  MCP7940::str = str;
  Wire.begin();
  Wire.beginTransmission(RTC_ADR);
  Wire.write(0x07); //cmd register in RTC
  Wire.write(0x40); //no alarm, output 1HZ signal
  Wire.endTransmission(true); //send queued data and stop transmission, releasing the bus.
  getTime();
  Wire.beginTransmission(RTC_ADR);
  Wire.write(0x00); //sec register in RTC
  Wire.write(0x80 + ((time.sec/10) << 4) + (time.sec % 10)); //no alarm, output 1HZ signal
  Wire.endTransmission(true); //send queued data and stop transmission, releasing the bus.
}

/*----------------------------
Func: getTime()
Desc: gets actual time and stores in time-structure
Vars: ---
------------------------------*/
void MCP7940::getTime(void)
{
  Wire.beginTransmission(RTC_ADR);
  Wire.write(0x00); //Sec register in RTC 
  Wire.endTransmission(true); //send queued data and stop transmission, releasing the bus
  Wire.requestFrom(RTC_ADR, 7, true);
  
  time.sec  = Wire.read() & 0x7F;
  time.sec  = (time.sec >> 4) * 10 + (time.sec & 0x0F);
  time.min  = Wire.read()& 0x7F;
  time.min  = (time.min >> 4) * 10 + (time.min & 0x0F);
  time.hour = Wire.read() & 0x3F;
  time.hour = (time.hour >> 4) * 10 + (time.hour & 0x0F);
  //Tag
  time.dayofweek = (Wire.read() & 0x07) - 1; //daycount in RTC 1..7, needed 0..6
  //Datum
  time.day  = Wire.read() & 0x3F;
  time.day  = (time.day >> 4) * 10 + (time.day & 0x0F);
  time.month= Wire.read() & 0x1F;
  time.month= (time.month >> 4) * 10 + (time.month & 0x0F);
  time.year = Wire.read();
  time.year = (time.year >> 4) * 10 + (time.year & 0x0F) + 2000;
}
/*----------------------------
Func: datestring()
Desc: gets actual time and retunrs datestring
Vars: ---
------------------------------*/
char *MCP7940::datestring(void)
{
  getTime();
  return datestring(time);
}

/*----------------------------
Func: datestring()
Desc: user time -> retunrs datestring
Vars: RTCTIME time
------------------------------*/
char *MCP7940::datestring(RTCTIME time)
{
  unsigned char i = 0;
   
  str[i++] = time.hour/10 + '0';
  str[i++] = time.hour%10 + '0';
  str[i++] = ':';
  str[i++] = time.min/10 + '0';
  str[i++] = time.min%10  + '0';
  str[i++] = ':';
  str[i++] = time.sec/10 + '0';
  str[i++] = time.sec%10 + '0'; 
  
  str[i++] = ' ';
  
  /* //print Weekday
  switch(time.dayofweek)
  {
    case 0:
     datestr[i++] = 'M';  datestr[i++] = 'o'; break;
    case 1:
     datestr[i++] = 'T';  datestr[i++] = 'u'; break;
    case 2:
     datestr[i++] = 'W';  datestr[i++] = 'e'; break;
    case 3:
     datestr[i++] = 'T';  datestr[i++] = 'h'; break;
    case 4:
     datestr[i++] = 'F';  datestr[i++] = 'r'; break;
    case 5:
     datestr[i++] = 'S';  datestr[i++] = 'a'; break;
    case 6:
     datestr[i++] = 'S';  datestr[i++] = 'u'; break;
  }

  datestr[i++] = ' ';
 */
 //calculate date
  if(time.day/10)
    str[i++] = time.day/10 + '0';
  str[i++] = time.day%10 + '0'; 
  str[i++] = '.';
  if(time.month/10)
    str[i++] = time.month/10 + '0';
  str[i++] = time.month%10 + '0'; 
  str[i++] = '.';
  time.year %= 1000;
  time.year %= 100;
  str[i++] = time.year/10 + '0';
  str[i++] = time.year%10 + '0'; 

  str[i] = 0;

  return str;
}
/*----------------------------
Func: timestring()
Desc: gets actual time and retunrs timestring
Vars: ---
------------------------------*/
char *MCP7940::timestring(void)
{
  getTime();
  return timestring(time);
}
/*----------------------------
Func: datestring()
Desc: user time -> retunrs timestring
Vars: RTCTIME time
------------------------------*/
char *MCP7940::timestring(RTCTIME time)
{
  unsigned char i = 0;
   
  str[i++] = time.hour/10 + '0';
  str[i++] = time.hour%10 + '0';
  str[i++] = ':';
  str[i++] = time.min/10 + '0';
  str[i++] = time.min%10  + '0';
  str[i++] = ':';
  str[i++] = time.sec/10 + '0';
  str[i++] = time.sec%10 + '0'; 
    
  str[i] = 0;

  return str;
}
/*----------------------------
Func: setTime()
Desc: user time -> set time in MCP7940
Vars: RTCTIME time
------------------------------*/
void MCP7940::setTime(RTCTIME *time)
{

  //Data-Check
  if(time->sec   > 59) time->sec   = 0;
  if(time->min   > 59) time->min   = 0;
  if(time->hour  > 23) time->hour  = 0;
  if(time->day  > 31)  time->day   = 1;
  if(time->month > 31) time->month = 1;
  if(time->year < 100) time->year += 2000;
   else if(time->year<2000 || time->year>=3000) time->year  = 2015;
  if(time->dayofweek > 7) time->dayofweek = 0;
  
  time->dayofweek += 1; //daycount in RTC 1..7, we use 0..6
  Wire.beginTransmission(RTC_ADR);
  Wire.write(0x00); //sec register in RTC
  //Zeit
  Wire.write(0);  //Chrystal off, set seconds later
  Wire.write(((time->min/10) << 4) + (time->min % 10));
  Wire.write(((time->hour/10) << 4) + (time->hour % 10)); //24h format
  //Day
  Wire.write(time->dayofweek + 0x08 + 0x10);  //OSCON(1 read only) VBAT (0 -> clear time stamp register) VBATEN (use bat)
  //Date
  Wire.write(((time->day/10) << 4) + (time->day % 10));
  Wire.write(((time->month/10) << 4) + (time->month % 10));
  Wire.write((((time->year-2000)/10) << 4) + ((time->year-2000) % 10));
  Wire.endTransmission(true);
  //Set seconds and start chrystal
  Wire.beginTransmission(RTC_ADR);
  Wire.write(0x00); //sec register in RTC
  Wire.write(0x80 + ((time->sec/10) << 4) + (time->sec % 10)); //no alarm, output 1HZ signal
  Wire.endTransmission(true);
}
